@@ -48,7 +48,7 @@ def listWorksheetInfo(presModel):
         if ("worksheet" in zones[z])
         and ("presModelHolder" in zones[z])
         and ("visual" in zones[z]["presModelHolder"])
-        and ("vizData" in zones[z]["presModelHolder"]["visual"])
+        # and ("vizData" in zones[z]["presModelHolder"]["visual"])
     ]
 
 
@@ -226,7 +226,7 @@ def getIndicesInfoStoryPoint(presModel, worksheet, noSelectFilter=True, noFieldC
 
 def getDataFull(presModelMap, originSegments):
     dataSegments = {}
-    if "dataDictionary" in presModelMap:
+    if ("dataDictionary" in presModelMap) and ("presModelHolder" in presModelMap["dataDictionary"]):
         dataSegments = presModelMap["dataDictionary"]["presModelHolder"]["genDataDictionaryPresModel"]["dataSegments"]
 
     dataSegmentscp = copy.deepcopy(dataSegments)
@@ -253,36 +253,46 @@ def onDataValue(it, value, cstring):
 
 
 def getData(dataFull, indicesInfo):
-    cstring = dataFull["cstring"]
+    if "cstring" in dataFull:
+        cstring = dataFull["cstring"]
+    else:
+        cstring = {}
     frameData = {}
     for index in indicesInfo:
         if index["dataType"] in dataFull:
             t = dataFull[index["dataType"]]
             if len(index["valueIndices"]) > 0:
-                frameData[f'{index["fieldCaption"]}-value'] = [
-                    onDataValue(it, t, cstring) for it in index["valueIndices"]
-                ]
+                values = []
+                for value in index["valueIndices"]:
+                    if value < len(t):
+                        values.append(onDataValue(value, t, cstring))
+                frameData[f'{index["fieldCaption"]}-value'] = values
             if len(index["aliasIndices"]) > 0:
-                frameData[f'{index["fieldCaption"]}-alias'] = [
-                    onDataValue(it, t, cstring) for it in index["aliasIndices"]
-                ]
+                values = []
+                for value in index["aliasIndices"]:
+                    if value < len(t):
+                        values.append(onDataValue(value, t, cstring))
+                frameData[f'{index["fieldCaption"]}-alias'] = values
         else:
             # if datatype is not found, try cstring
             t = cstring
             if len(index["valueIndices"]) > 0:
-                frameData[f'{index["fieldCaption"]}-value'] = [
-                    onDataValue(it, t, cstring) for it in index["valueIndices"]
-                ]
+                values = []
+                for value in index["valueIndices"]:
+                    if value < len(t):
+                        values.append(onDataValue(value, t, cstring))
+                frameData[f'{index["fieldCaption"]}-value'] = values
             if len(index["aliasIndices"]) > 0:
-                frameData[f'{index["fieldCaption"]}-alias'] = [
-                    onDataValue(it, t, cstring) for it in index["aliasIndices"]
-                ]
+                values = []
+                for value in index["aliasIndices"]:
+                    if value < len(t):
+                        values.append(onDataValue(value, t, cstring))
+                frameData[f'{index["fieldCaption"]}-alias'] = values
     return frameData
 
 
-def getDataFullCmdResponse(presModel, originSegments):
-    dataSegments = {}
-    if "dataDictionary" in presModel:
+def getDataFullCmdResponse(presModel, originSegments, dataSegments={}):
+    if (not dataSegments) and ("dataDictionary" in presModel):
         dataSegments = presModel["dataDictionary"]["dataSegments"]
     dataSegmentscp = copy.deepcopy(dataSegments)
     originSegmentscp = copy.deepcopy(originSegments)
@@ -397,6 +407,20 @@ def getWorksheetCmdResponse(selectedZone, dataFull):
     return getData(dataFull, result)
 
 
+def getWorksheetDownloadCmdResponse(dataFull, underlyingDataTableColumns):
+    result = [
+        {
+            "fieldCaption": t["fieldCaption"],
+            "valueIndices": t["valueIndices"],
+            "aliasIndices": t["aliasIndices"],
+            "dataType": t["dataType"]
+        }
+        for t in underlyingDataTableColumns
+        if t.get("fieldCaption")
+    ]
+    return getData(dataFull, result)
+
+
 def selectWorksheetCmdResponse(presModel, logger):
     zonesWithWorksheet = listWorksheetCmdResponse(presModel)
 
